define class DBFListener as _ReportListener ;
	of home() + 'ffc\_reportlistener.vcx'
	cOutputDBF       = ''
		&& The name of the table to output to
	cOutputAlias     = ''
		&& The alias for the cursor to output to
	lUseCursor       = .T.
		&& .T. to use a cursor, .F. for a table
	lDeleteOnDestroy = .T.
		&& .T. to delete the table when this object is destroyed
	ListenerType     = 3

* Just before the report is run, create a cursor to hold the rendered report
* contents.

	function BeforeReport
		local lcTable
		with This

* If a table name and/or an alias wasn't specified, create default names.

			if empty(.cOutputDBF)
				.cOutputDBF = addbs(sys(2023)) + sys(2015) + '.dbf'
			endif empty(.cOutputDBF)
			if empty(.cOutputAlias)
				.cOutputAlias = strtran(juststem(.cOutputDBF), ' ', '_')
			endif empty(.cOutputAlias)

* If the cursor is already open it, close it. If the table already exists,
* nuke it.

			use in select (.cOutputAlias)
			if file(.cOutputDBF)
				erase (.cOutputDBF)
				erase forceext(.cOutputDBF, 'FPT')
				erase forceext(.cOutputDBF, 'CDX')
			endif file(.cOutputDBF)

* Create either a cursor or a table.

			lcTable = iif(.lUseCursor, 'cursor ' + .cOutputAlias, ;
				'table ' + .cOutputDBF)
			create &lcTable (FRXRECNO I, OBJTYPE I, OBJCODE I, LEFT I, TOP I, ;
				WIDTH I, HEIGHT I, CONTTYPE I, CONTENTS M nocptrans, PAGE I)
			index on PAGE tag PAGE
		endwith

* Do the usual behavior.

		dodefault()
	endfunc

* As each object is rendered, add a record to the cursor. This code doesn't
* currently do anything special with images because I haven't decided what to
* do with them yet.

	function Render(tnFRXRecNo, tnLeft, tnTop, tnWidth, tnHeight, ;
		tnObjectContinuationType, tcContentsToBeRendered, tiGDIPlusImage)
		local lcContents, ;
			liObjType, ;
			liObjCode
		with This
			if empty(tcContentsToBeRendered)
				lcContents = ''
			else
				lcContents = strconv(tcContentsToBeRendered, 6)
			endif empty(tcContentsToBeRendered)
			.SetFRXDataSession()
			go tnFRXRecno in FRX
			liObjType = FRX.OBJTYPE
			liObjCode = FRX.OBJCODE
			.ResetDataSession()
			insert into (.cOutputAlias) values (tnFRXRecNo, liObjType, ;
				liObjCode, tnLeft, tnTop, tnWidth, tnHeight, ;
				tnObjectContinuationType, lcContents, .PageNo)
		endwith
	endfunc	

* Close the cursor and delete it if we're supposed to when this object is
* destroyed.

	function Destroy
		with This
			if not empty(.cOutputAlias)
				use in select(.cOutputAlias)
				if .lDeleteOnDestroy
					erase (.cOutputDBF)
					erase forceext(.cOutputDBF, 'FPT')
					erase forceext(.cOutputDBF, 'CDX')
				endif .lDeleteOnDestroy
			endif not empty(.cOutputAlias)
		endwith
	endfunc
enddefine
